@@ -26,73 +26,64 @@ from.
 Also radian is the default. Degree can be used, but generally the default is
 to assume radian.
 """
+from astropy import units as u
 from astropy.units import rad, deg, hourangle  # noqa
-from astropy.coordinates import (EarthLocation, SkyCoord, AltAz, Longitude,
-                                 Latitude, get_sun)
+from astropy.coordinates import (
+    EarthLocation,
+    SkyCoord,
+    AltAz,
+    Longitude,
+    Latitude,
+    get_sun,
+    get_moon,
+)
 import astropy.time
 import numpy as np
 
-from km3pipe.math import hsin, space_angle
-
 from km3astro.constants import (
-    arca_longitude, arca_latitude, arca_height,
-    orca_longitude, orca_latitude, orca_height,
-    antares_longitude, antares_latitude, antares_height,
+    arca_longitude,
+    arca_latitude,
+    arca_height,
+    orca_longitude,
+    orca_latitude,
+    orca_height,
+    antares_longitude,
+    antares_latitude,
+    antares_height,
 )
 from km3astro.time import np_to_astrotime
 from km3astro.random import random_date, random_azimuth
-
-
-# (right ascension, declination)
-SIRIUS = SkyCoord('06 45 08.9 -16 42 58', unit=(hourangle, deg))
-CANOPUS = SkyCoord('06 23 57.1 -52 41 45', unit=(hourangle, deg))
-ARCTURUS = SkyCoord('14 15 39.7 +19 10 57', unit=(hourangle, deg))
-ANTARES = SkyCoord('16 29 24.4 -26 25 55', unit=(hourangle, deg))
-
-RX_J1713 = SkyCoord('17 17 33.6 -39 45 36.4', unit=(hourangle, deg))
-VELA_X = SkyCoord('08 35 00 -45 36 00', unit=(hourangle, deg))
-
-# Sagittarius A* (sagittarius a-star)
-# GALACTIC_CENTER = SkyCoord(0 * deg, 0 * deg, frame='galactic').icrs
-SAGITTARIUS_A_STAR = SkyCoord('17 45 40.0409 -29 0 28.118', unit=(hourangle, deg))      # noqa
-GALACTIC_CENTER = SAGITTARIUS_A_STAR
+from km3astro.sources import GALACTIC_CENTER
 
 
 LOCATIONS = {
-    'arca': EarthLocation.from_geodetic(
+    "arca": EarthLocation.from_geodetic(
         lon=Longitude(arca_longitude * deg),
         lat=Latitude(arca_latitude * deg),
-        height=arca_height
+        height=arca_height,
     ),
-    'orca': EarthLocation.from_geodetic(
+    "orca": EarthLocation.from_geodetic(
         lon=Longitude(orca_longitude * deg),
         lat=Latitude(orca_latitude * deg),
-        height=orca_height
+        height=orca_height,
     ),
-    'antares': EarthLocation.from_geodetic(
+    "antares": EarthLocation.from_geodetic(
         lon=Longitude(antares_longitude * deg),
         lat=Latitude(antares_latitude * deg),
-        height=antares_height
+        height=antares_height,
     ),
 }
 
 
-def get_location(location='orca'):
-    try:
-        loc = LOCATIONS[location]
-    except KeyError:
-        raise KeyError("Invalid location, valid are 'orca', 'arca', 'antares'")
-    return loc
-
-
 def neutrino_to_source_direction(phi, theta, radian=True):
     """Flip the direction.
 
     Parameters
-    ==========
+    ----------
     phi, theta: neutrino direction
     radian: bool [default=True]
         receive + return angles in radian? (if false, use degree)
+
     """
     phi = np.atleast_1d(phi).copy()
     theta = np.atleast_1d(theta).copy()
@@ -110,6 +101,18 @@ def neutrino_to_source_direction(phi, theta, radian=True):
 
 
 def source_to_neutrino_direction(azimuth, zenith, radian=True):
+    """Flip the direction.
+
+    Parameters
+    ----------
+    zenith : float
+        neutrino origin
+    azimuth: float
+        neutrino origin
+    radian: bool [default=True]
+        receive + return angles in radian? (if false, use degree)
+
+    """
     azimuth = np.atleast_1d(azimuth).copy()
     zenith = np.atleast_1d(zenith).copy()
     if not radian:
@@ -123,14 +126,31 @@ def source_to_neutrino_direction(azimuth, zenith, radian=True):
     return phi, theta
 
 
+def get_location(location):
+    try:
+        loc = LOCATIONS[location]
+    except KeyError:
+        raise KeyError("Invalid location, valid are 'orca', 'arca', 'antares'")
+    return loc
+
+
 def Sun(time):
+    """Wrapper around astropy's get_sun, accepting numpy/pandas time objects."""
     if not isinstance(time, astropy.time.Time):
         # if np.datetime64, convert to astro time
         time = np_to_astrotime(time)
     return get_sun(time)
 
 
-def local_frame(time, location='orca'):
+def Moon(time):
+    """Wrapper around astropy's get_moon, accepting numpy/pandas time objects."""
+    if not isinstance(time, astropy.time.Time):
+        # if np.datetime64, convert to astro time
+        time = np_to_astrotime(time)
+    return get_moon(time)
+
+
+def local_frame(time, location):
     """Get the (horizontal) coordinate frame of your detector."""
     if not isinstance(time, astropy.time.Time):
         # if np.datetime64, convert to astro time
@@ -140,8 +160,7 @@ def local_frame(time, location='orca'):
     return frame
 
 
-def local_event(azimuth, time, zenith, radian=True,
-                location='orca', **kwargs):
+def local_event(azimuth, time, zenith, location, radian=True, **kwargs):
     """Create astropy events from detector coordinates."""
     zenith = np.atleast_1d(zenith).copy()
     azimuth = np.atleast_1d(azimuth).copy()
@@ -149,29 +168,42 @@ def local_event(azimuth, time, zenith, radian=True,
         azimuth *= np.pi / 180
         zenith *= np.pi / 180
     altitude = zenith - np.pi / 2
+
+    loc = get_location(location)
     # neutrino telescopes call the co-azimuth "azimuth"
-    true_azimuth = (np.pi / 2 - azimuth) % np.pi
+    true_azimuth = (
+        np.pi / 2 - azimuth + np.pi + convergence_angle(loc.lat.rad, loc.lon.rad)
+    ) % (2 * np.pi)
     frame = local_frame(time, location=location)
-    event = SkyCoord(alt=altitude * rad, az=true_azimuth * rad, frame=frame,
-                     **kwargs)
+    event = SkyCoord(alt=altitude * rad, az=true_azimuth * rad, frame=frame, **kwargs)
     return event
 
 
-def sun_local(time, loc='orca'):
-    frame = local_frame(time, location='orca')
-    sun = get_sun(time)
+def sun_local(time, loc):
+    """Sun position in local coordinates."""
+    frame = local_frame(time, location=loc)
+    sun = Sun(time)
     sun_local = sun.transform_to(frame)
     return sun_local
 
 
-def gc_in_local(time, loc='orca'):
-    frame = local_frame(time, location='orca')
+def moon_local(time, loc):
+    """Moon position in local coordinates."""
+    frame = local_frame(time, location=loc)
+    moon = Moon(time)
+    moon_local = moon.transform_to(frame)
+    return moon_local
+
+
+def gc_in_local(time, loc):
+    """Galactic center position in local coordinates."""
+    frame = local_frame(time, location=loc)
     gc = GALACTIC_CENTER
     gc_local = gc.transform_to(frame)
     return gc_local
 
 
-def orca_gc_dist(azimuth, time, zenith, frame='detector'):
+def orca_gc_dist(azimuth, time, zenith, frame="detector"):
     """Return angular distance of event to GC.
 
     Parameters
@@ -180,10 +212,10 @@ def orca_gc_dist(azimuth, time, zenith, frame='detector'):
         valid are 'detector', 'galactic', 'icrs', 'gcrs'
     """
     evt = local_event(azimuth, time, zenith)
-    galcen = gc_in_local(time, loc='orca')
-    if frame == 'detector':
+    galcen = gc_in_local(time, loc)
+    if frame == "detector":
         pass
-    elif frame in ('galactic', 'icrs', 'gcrs'):
+    elif frame in ("galactic", "icrs", "gcrs"):
         evt = evt.transform_to(frame)
         galcen = galcen.transform_to(frame)
     return evt.separation(galcen).radian
@@ -192,12 +224,12 @@ def orca_gc_dist(azimuth, time, zenith, frame='detector'):
 def orca_sun_dist(azimuth, time, zenith):
     """Return distance of event to sun, in detector coordinates."""
     evt = local_event(azimuth, time, zenith)
-    sun = sun_local(time, loc='orca')
+    sun = sun_local(time, loc)
     dist = evt.separation(sun).radian
     return dist
 
 
-def gc_dist_random(zenith, frame='detector'):
+def gc_dist_random(zenith, frame="detector"):
     """Generate random (time, azimuth) events and get distance to GC."""
     n_evts = len(zenith)
     time = random_date(n=n_evts)
@@ -216,7 +248,7 @@ def sun_dist_random(zenith):
 
 
 class Event(object):
-    def __init__(self, zenith, azimuth, time, location='orca'):
+    def __init__(self, zenith, azimuth, time, location):
         self.zenith = zenith
         self.azimuth = azimuth
         self.time = time
@@ -228,3 +260,88 @@ class Event(object):
         azimuth = random_azimuth(n_evts)
         time = random_date(n_evts)
         return cls(zenith, azimuth, time, **initargs)
+
+
+def convergence_angle(lat, lon):
+    """Calculate the converge angle on the UTM grid.
+
+    Parameters
+    ----------
+    lon : number
+        Longitude in rad
+    lat : number
+        Latitude in rad
+
+    """
+    latitude_deg = lat * u.deg
+
+    if latitude_deg > 84 * u.deg or latitude_deg < -80 * u.deg:
+        raise ValueError(
+            "UTM coordinate system is only defined between -80deg S and 84deg N."
+        )
+
+    # detector position, longitude and latitude in rad
+    # lambda  = longitude
+    phi = lat
+
+    # find UTM zone and central meridian
+
+    # longitude of the central meridian of UTM zone in rad
+    lambda0 = longitude_of_central_meridian(utm_zone(lon))
+    omega = lon - lambda0
+
+    # parameters of the Earth ellipsoid
+    sma = 6378137  # semi-major axis in meters (WGS84)
+    ecc = 0.0066943800  # eccentricity (WGS84)
+
+    rho = sma * (1 - ecc) / pow(1 - ecc * np.sin(phi) ** 2, 3 / 2)
+    nu = sma / np.sqrt(1 - ecc * np.sin(phi) ** 2)
+    psi = nu / rho
+    t = np.tan(phi)
+
+    angle = (
+        np.sin(phi) * omega
+        - np.sin(phi) * omega ** 3 / 3 * pow(np.cos(phi), 2) * (2 * psi ** 2 - psi)
+        - np.sin(phi)
+        * omega ** 5
+        / 15
+        * pow(np.cos(phi), 4)
+        * (
+            psi ** 4 * (11 - 24 * t ** 2)
+            - psi ** 3 * (11 - 36 * t ** 2)
+            + 2 * psi ** 2 * (1 - 7 * t ** 2)
+            + psi * t ** 2
+        )
+        - np.sin(phi)
+        * omega ** 7
+        / 315
+        * pow(np.cos(phi), 6)
+        * (17 - 26 * t ** 2 + 2 * t ** 4)
+    )
+
+    return angle
+
+
+def utm_zone(lat):
+    """The UTM zone for a given latitude
+
+    Parameters
+    ----------
+    lat : number
+        Latitude in rad
+
+    """
+    return 1 + int((np.pi + lat) / (6 * np.pi / 180))
+
+
+def longitude_of_central_meridian(utmzone):
+    """The longitude of the central meridian for a given UTM zone.
+
+    Parameters
+    ----------
+    utmzone : number
+        The UTM zone.
+
+    """
+    zone_width = 6 * np.pi / 180
+    return -np.pi + (utmzone - 1) * zone_width + zone_width / 2
